//*******************************************************************************
// file  cam_init.cpp
//
// attention
// Copyright (C) 2020 MACNICA,Inc. All Rights Reserved.\n
//   This software is licensed "AS IS".
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/utsname.h>
#include <linux/i2c-dev.h>
#include <linux/i2c.h>
#include <sys/ioctl.h>

#include "ov5642.h"

#define OV5642_I2C_ADDR (0x3C)
#define UB934_I2C_ADDR (0x30)
#define UB933_I2C_ADDR (0x58)
#define SER_I2C_ADDR (UB933_I2C_ADDR+1)

u_int8_t des_i2c_read(u_int8_t reg_addr, u_int8_t* data);
u_int8_t des_i2c_write(u_int8_t reg_addr, u_int8_t data);
u_int8_t ser_i2c_read(u_int8_t reg_addr, u_int8_t* data);
u_int8_t ser_i2c_write(u_int8_t reg_addr, u_int8_t data);
u_int8_t cam_i2c_read(u_int16_t reg_addr, u_int8_t* data);
u_int8_t cam_i2c_write(u_int16_t reg_addr, u_int8_t data);

void serdes_init(void);
void ov5642_init(void);

void serdes_init(void)
{
    u_int8_t rdata;

    des_i2c_read(0x00, &rdata); //I2C device ID
    des_i2c_read(0x04, &rdata); //device status
    des_i2c_write(0x01, 0x07);  //device reset
    usleep(1000);
    des_i2c_read(0x04, &rdata); //device status

    des_i2c_write(0x4c, 0x01);  //port0 write enable
    des_i2c_read(0x5b, &rdata);
    printf("ds90ub934 remote serializer ID=0x%02x\n", rdata);

    des_i2c_write(0x58, 0x58);  // Pass-through enabled
    des_i2c_write(0x5c, SER_I2C_ADDR<<1); // ser alias ID
    des_i2c_write(0x5d, OV5642_I2C_ADDR<<1); // slave ID 0
    //des_i2c_write(0x65, OV5642_I2C_ADDR<<1|0x01); // slave alias ID 0, auto ack
    des_i2c_write(0x65, OV5642_I2C_ADDR<<1); // slave alias ID 0

    //des_i2c_read(0x6d, &rdata); //PORT_CONFIG
    des_i2c_write(0x6d, 0x02); //mode override, RAW12HF

    des_i2c_read(0x4d, &rdata);

    ser_i2c_read(0x00, &rdata); //I2C device ID
    ser_i2c_write(0x01, 0x33); //device reset
    usleep(1000);

    ser_i2c_read(0x0c, &rdata); //status
    ser_i2c_write(0x05, 0x22); //mode override, 12-bit mode
    ser_i2c_read(0x05, &rdata); //mode select
    ser_i2c_write(0x11, 0x64); //SCL High Time=5us
    ser_i2c_write(0x12, 0x64); //SCL Low Time=5us
    ser_i2c_read(0x0c, &rdata); //status
}

void ov5642_init()
{
    int i;

    for(i = 0;ov5642[i].adrs != 0xFFFF;i ++)
    {
        cam_i2c_write(ov5642[i].adrs, ov5642[i].data);
    }
}

u_int8_t i2c_read_8bit(u_int8_t slv_addr, u_int8_t reg_addr, u_int8_t* data)
{
    int32_t fd;
    u_int8_t txbuf[1];
    struct i2c_msg msg[2];
    struct i2c_rdwr_ioctl_data packes;

    fd = open("/dev/i2c-0", O_RDWR|O_SYNC);
    if(fd < 0){
        printf("i2c device file open error!\n");
        return -1;
    }

    txbuf[0] = reg_addr;

    msg[0].addr = slv_addr;
    msg[0].flags = 0,
    msg[0].len = 1,
    msg[0].buf = txbuf;

    msg[1].addr = slv_addr;
    msg[1].flags = I2C_M_RD;
    msg[1].len = 1;
    msg[1].buf = data;

    packes.msgs = msg;
    packes.nmsgs = 2;

    if(ioctl(fd, I2C_RDWR, &packes) < 0){
        printf("i2c_read_8bit: failed to ioctl\n");
        close(fd);
        return -1;
    }

    close(fd);
    return 0;
}


u_int8_t i2c_write_8bit(u_int8_t slv_addr, u_int8_t reg_addr, u_int8_t data)
{
    int32_t fd;
    u_int8_t txbuf[2];
    struct i2c_msg msg[1];
    struct i2c_rdwr_ioctl_data packes;

    fd = open("/dev/i2c-0", O_RDWR|O_SYNC);
    if(fd < 0){
        printf("i2c device file open error!\n");
        return -1;
    }

    txbuf[0] = reg_addr;
    txbuf[1] = data;

    msg[0].addr = slv_addr;
    msg[0].flags = 0,
    msg[0].len = 2,
    msg[0].buf = txbuf;

    packes.msgs = msg;
    packes.nmsgs = 1;

    if(ioctl(fd, I2C_RDWR, &packes) < 0){
        printf("i2c_write_8bit: failed to ioctl\n");
        close(fd);
        return -1;
    }

    close(fd);
    return 0;

}

u_int8_t i2c_read_16bit(u_int8_t slv_addr, u_int16_t reg_addr, u_int8_t* data)
{
    int32_t fd;
    u_int8_t txbuf[2];
    struct i2c_msg msg[2];
    struct i2c_rdwr_ioctl_data packes;

    fd = open("/dev/i2c-0", O_RDWR|O_SYNC);
    if(fd < 0){
        printf("i2c device file open error!\n");
        return -1;
    }

    txbuf[0] = (reg_addr & 0xff00) >> 8;
    txbuf[1] = (reg_addr & 0x00ff);

    msg[0].addr = slv_addr;
    msg[0].flags = 0,
    msg[0].len = 2,
    msg[0].buf = txbuf;

    msg[1].addr = slv_addr;
    msg[1].flags = I2C_M_RD;
    msg[1].len = 1;
    msg[1].buf = data;

    packes.msgs = msg;
    packes.nmsgs = 2;

    if(ioctl(fd, I2C_RDWR, &packes) < 0){
        printf("i2c_read_16bit: failed to ioctl\n");
        close(fd);
        return -1;
    }

    close(fd);
    return 0;
}

u_int8_t i2c_write_16bit(u_int8_t slv_addr, u_int16_t reg_addr, u_int8_t data)
{
    int32_t fd;
    u_int8_t txbuf[3];
    struct i2c_msg msg[1];
    struct i2c_rdwr_ioctl_data packes;

    fd = open("/dev/i2c-0", O_RDWR|O_SYNC);
    if(fd < 0){
        printf("i2c device file open error!\n");
        return -1;
    }

    txbuf[0] = (reg_addr & 0xff00) >> 8;
    txbuf[1] = (reg_addr & 0x00ff);
    txbuf[2] = data;

    msg[0].addr = slv_addr;
    msg[0].flags = 0,
    msg[0].len = 3,
    msg[0].buf = txbuf;

    packes.msgs = msg;
    packes.nmsgs = 1;

    if(ioctl(fd, I2C_RDWR, &packes) < 0){
        printf("i2c_write_16bit: failed to ioctl\n");
        close(fd);
        return -1;
    }

    close(fd);
    return 0;

}

u_int8_t cam_i2c_read(u_int16_t reg_addr, u_int8_t* data)
{
    i2c_read_16bit(OV5642_I2C_ADDR, reg_addr, data);
//  printf("Register read ov5642 address=0x%04x data=0x%02x\n", reg_addr, *data);

    return 0;
}

u_int8_t cam_i2c_write(u_int16_t reg_addr, u_int8_t data)
{
    i2c_write_16bit(OV5642_I2C_ADDR, reg_addr, data);
//  printf("Register write ov5642 address=0x%04x data=0x%02x\n", reg_addr, data);

    return 0;
}

u_int8_t ser_i2c_read(u_int8_t reg_addr, u_int8_t* data)
{
    i2c_read_8bit(SER_I2C_ADDR, reg_addr, data);
//  printf("Register read ds90ub933 address=0x%02x data=0x%02x\n", reg_addr, *data);

    return 0;
}

u_int8_t ser_i2c_write(u_int8_t reg_addr, u_int8_t data)
{
    i2c_write_8bit(SER_I2C_ADDR, reg_addr, data);
//  printf("Register write ds90ub933 address=0x%02x data=0x%02x\n", reg_addr, data);

    return 0;
}

u_int8_t des_i2c_read(u_int8_t reg_addr, u_int8_t* data)
{
    i2c_read_8bit(UB934_I2C_ADDR, reg_addr, data);
//  printf("Register read ds90ub934 address=0x%02x data=0x%02x\n", reg_addr, *data);

    return 0;
}

u_int8_t des_i2c_write(u_int8_t reg_addr, u_int8_t data)
{
    i2c_write_8bit(UB934_I2C_ADDR, reg_addr, data);
//  printf("Register write ds90ub934 address=0x%02x data=0x%02x\n", reg_addr, data);

    return 0;
}
